import 'package:flutter_webrtc/media_stream.dart';

abstract class CallSession<S> {
  LocalStreamCallback onLocalStreamReceived;
  RemoteStreamCallback<S> onRemoteStreamReceived;
  RemoteStreamCallback<S> onRemoteStreamRemoved;
  UserConnectionStateCallback<S> onUserNoAnswer;
  UserActionCallback<S> onCallAcceptedByUser;
  UserActionCallback<S> onCallRejectedByUser;
  UserActionCallback<S> onReceiveHungUpFromUser;
  SessionClosedCallback<S> onSessionClosed;

  void startCall([Map<String, String> userInfo]);

  void acceptCall([Map<String, String> userInfo]);

  void reject([Map<String, String> userInfo]);

  void hungUp([Map<String, String> userInfo]);

  void setMicrophoneMute(bool mute);

  void enableSpeakerphone(bool enable);

  void setVideoEnabled(bool enabled);

  ///Future contains isFrontCamera
  ///Throws error if switching camera failed
  Future<bool> switchCamera();

  RTCSessionState get state;

  int get callerId;

  String get sessionId;

  int get callType;

  Set<int> get opponentsIds;
}

typedef void LocalStreamCallback(MediaStream stream);
typedef void RemoteStreamCallback<T>(T session, int userId, MediaStream stream);
typedef void UserConnectionStateCallback<T>(T session, int userId);
typedef void UserActionCallback<T>(T session, int userId,
    [Map<String, String> userInfo]);
typedef void SessionClosedCallback<T>(T session);

enum RTCSessionState {
  RTC_SESSION_PENDING,
  RTC_SESSION_NEW,
  RTC_SESSION_CONNECTING,
  RTC_SESSION_CONNECTED,
  RTC_SESSION_GOING_TO_CLOSE,
  RTC_SESSION_CLOSED
}
